#!/bin/sh
# Script to automate installation of OpenSSH on Zaurus 5K series Linux PDA
# by maldoror@worldnet.att.net
# Must be run as root!
#
# V1.0		2/25/02
#	Initial release
# V1.1		2/28/02
#	Changes:
#	Fixed warning-banner copy problem
#	Added symlink for "/usr/local/bin/ssh" to fix sftp dependency
#	Added backup of config files prior to run. Lack of free space may lead to corruption of "/etc/inetd.conf" and "/etc/passwd", so we now back them up before doing anything.
#	Added removal of existing ssh binaries and symbolic links prior to installation.  This fixes problems when re-installing or upgrading.
#	Removes old ssh entries in /etc/inetd.conf to prevent problems cause by re-installing to a different destination
#	Stripped binaries to reduce size
#	Grep ssh out of netstat to verify restart of inetd at end of script
#	Added paranoid file permissioning of SSH files just in case anyone uses their Zaurus as a multi-user Unix server
#	Fixed bug that caused all three user keys to be RSA keys for SSH1
# V1.2		3/5/02
#	Changes:
#	Minor stdout redirection problem fixed.  This was potentially whacking the edited /etc/password file so that it only contains a root account entry.  This didn't pose a functional problem.
# V1.2.1	3/6/02
#	Changes:
#	Added check to make sure that the script is being run from the same directory as the extracted binary files (thanks to rwarner1@directvinternet.com).
#	Added symbolic link from /usr/local/etc/sshd_config to /etc/sshd_config.  This way, if you launch sshd from the command line, it will find the config file in the compile-in default location of /usr/local/etc/sshd_config (thanks to rwarner1@directvinternet.com).
# V1.2.2	3/7/02
#	Changes:
#	Updated binaries to OpenSSH 3.1-p1
#	Moved ssh config files from /etc to /etc/ssh
#	Made changes to reflect that /usr/local references are no longer required.  Openssh 3.1-p1 was properly compiled with /usr and /etc as the appropriate paths.
# V1.3		4/1/02
#	Changes:
#	Fix some stupid stuff like reverting user keys to default names
#	Fix bug related to config files not being referenced correctly in the new directory /etc/ssh
#	Moved search path update from .bashrc to .profile (more appropriate but no functional difference)
# V1.4		4/8/02
#	Changes:
#	Fixed bug that deletes all /etc/passwd entries except for root.  Don't worry, this bug has no effect on the Zaurus unless you are running it as a multi-user server :)
#	Updated zlib library to 1.1.4 this fixes a security problem with zlib 1.1.3
#	Fixed name of user ssh config file "~/.ssh/ssh_config" > "~/.ssh/config"
#	Added an install switch to specify client / client-server / full install
#	A few user-friendliness features added.
#	Added uninstall support.
#	README.Zaurus file has been vastly improved.
#	Package fixes possible permission bit problem during install time.
# V1.5		6/2/02
#	Changes:
#	Added fix to not overwrite existing SSH host keys when upgrading.
#	Upgraded OpenSSH to 3.2.3p1 compiled with OpenSSL 0.9.6d and zlib1.1.4
#	HTML man pages updated to 3.2.3p1 man pages 
#
# OpenSSH Configure /Compile switches:
#	./configure --prefix=/usr --sysconfdir=/etc --libexecdir=/usr/sbin --with-zlib=/home/guest/maldoror@worldnet.att.net/zlib-1.1.4 --with-ssl-dir=/home/guest/maldoror@worldnet.att.net/openssl-0.9.6d --without-skey --without-tcp-wrappers --without-pam --with-ipv4-default --with-cflags=-static;make
#
# ------------------------------------------------------------------------------

# Select install destination
if [ -z $1 ]; then
	echo "Usage: $0 Location Type"
	echo "  Location = m=Main Memory"
	echo "             c=Compact Flash"
	echo "             s=SD ram"
	echo ""
	echo "  Type     = c=Client only"
	echo "             s=Client and Server"
	echo "             f=Full Installation"
	echo "             u=Uninstall"
	echo ""
	echo "Note: Preferred installation is to SD RAM"
	echo "       since CF may be occupied by a "
	echo "       network adapter."
	echo "If no type is specified, the default is"
	echo "full."
	echo ""
	echo "Example: ./install.sh s f"
	echo "  Performs full install to sd ram."
	exit 1
fi

case "$1" in
"m")
	DESTINATION=main;
	PREFIX=/usr;;
"c")
	DESTINATION=cf;
	PREFIX=/mnt/cf;;
"s")
	DESTINATION=sd;
	PREFIX=/mnt/card;;
*)
	echo "Invalid install location specified. Aborting."
	exit 1;;
esac

case "$2" in
"c")
	UNINSTALL=0;
	CLIENT=1;
	SERVER=0;
	FULL=0;;
"s")
	UNINSTALL=0;
	CLIENT=1;
	SERVER=1;
	FULL=0;;
"f")
	UNINSTALL=0;
	CLIENT=1;
	SERVER=1;
	FULL=1;;
"u")
	UNINSTALL=1;
	CLIENT=0;
	SERVER=0;
	FULL=0;;
*)
	UNINSTALL=0;
	CLIENT=1;
	SERVER=1;
	FULL=1;;
esac

if [ $UNINSTALL = 1 ]; then
	echo "----- Uninstalling OpenSSH"
	echo " NOTE: All keys and configuration files"
	echo "       will be deleted!"
	rm -f /usr/sbin/sshd 1>/dev/null 2>&1
	rm -f /usr/sbin/sftp-server 1>/dev/null 2>&1
	rm -f /usr/bin/ssh 1>/dev/null 2>&1
	rm -f /usr/bin/ssh-keygen 1>/dev/null 2>&1
	rm -f /usr/bin/ssh-keyscan 1>/dev/null 2>&1
	rm -f /usr/bin/ssh-agent 1>/dev/null 2>&1
	rm -f /usr/bin/ssh-add 1>/dev/null 2>&1
	rm -f /usr/bin/sftp 1>/dev/null 2>&1
	rm -f /usr/bin/scp 1>/dev/null 2>&1
	rm -f $PREFIX/sbin/sshd 1>/dev/null 2>&1
	rm -f $PREFIX/sbin/sftp-server 1>/dev/null 2>&1
	rm -f $PREFIX/bin/ssh 1>/dev/null 2>&1
	rm -f $PREFIX/bin/ssh-keygen 1>/dev/null 2>&1
	rm -f $PREFIX/bin/ssh-keyscan 1>/dev/null 2>&1
	rm -f $PREFIX/bin/ssh-agent 1>/dev/null 2>&1
	rm -f $PREFIX/bin/ssh-add 1>/dev/null 2>&1
	rm -f $PREFIX/bin/sftp 1>/dev/null 2>&1
	rm -f $PREFIX/bin/scp 1>/dev/null 2>&1
	rm -rf /etc/ssh 1>/dev/null 2>&1
	rm -rf /home/root/.ssh 1>/dev/null 2>&1
	rm -rf /home/root/Documents/OpenSSH 1>/dev/null 2>&1
	echo ""
	echo "      Uninstall finished."
	exit 0
fi

# Verify that we are running this script from the same directory as the ssh binaries.  We only check for sshd.
if ! [ -f ./sshd ]; then
	echo "Please run this script from the same directory"
	echo "as the bundled binary files."
	exit 1
fi

# If the archive is unzippied on a Windows system, the execute permission bits may be screwed up, so we explicitly set it for all binaries. 
chmod +x ./sshd 1>/dev/null 2>&1
chmod +x ./sftp-server 1>/dev/null 2>&1
chmod +x ./ssh 1>/dev/null 2>&1
chmod +x ./ssh-keygen 1>/dev/null 2>&1
chmod +x ./ssh-keyscan 1>/dev/null 2>&1
chmod +x ./ssh-agent 1>/dev/null 2>&1
chmod +x ./ssh-add 1>/dev/null 2>&1
chmod +x ./sftp 1>/dev/null 2>&1
chmod +x ./scp 1>/dev/null 2>&1

echo "--- Installing OpenSSH ---"

if [ $CLIENT = 1 ]; then
	mkdir /home/root/.ssh 1>/dev/null 2>&1
fi

if [ $SERVER = 1 ]; then
	mkdir /etc/ssh 1>/dev/null 2>&1
fi

if [ $DESTINATION = cf ]; then
	mkdir /mnt/cf/bin 1>/dev/null 2>&1
	mkdir /mnt/cf/sbin 1>/dev/null 2>&1
fi

if [ $DESTINATION = sd ]; then
	mkdir /mnt/card/bin 1>/dev/null 2>&1
	mkdir /mnt/card/sbin 1>/dev/null 2>&1
fi

if [ $SERVER = 1 ]; then
	echo ""
	echo "----- Backing up original config files"
	echo "       /etc/inetd  --> /etc/inetd.backup"
	cp /etc/inetd /etc/inetd.backup 1>/dev/null 2>&1
	echo "       /etc/passwd --> /etc/passwd.backup"
	cp /etc/passwd /etc/passwd.backup 1>/dev/null 2>&1
fi

echo ""
echo "----- Deleting old ssh binaries and links"
echo "      Preserving old conf files and keys"
rm -f /usr/sbin/sshd 1>/dev/null 2>&1
rm -f /usr/sbin/sftp-server 1>/dev/null 2>&1
rm -f /usr/bin/ssh 1>/dev/null 2>&1
rm -f /usr/bin/ssh-keygen 1>/dev/null 2>&1
rm -f /usr/bin/ssh-keyscan 1>/dev/null 2>&1
rm -f /usr/bin/ssh-agent 1>/dev/null 2>&1
rm -f /usr/bin/ssh-add 1>/dev/null 2>&1
rm -f /usr/bin/sftp 1>/dev/null 2>&1
rm -f /usr/bin/scp 1>/dev/null 2>&1

# Check to see if sshd is already defined in inetd.conf.  If so, then skip next step.
if [ $SERVER = 1 ]; then
	echo ""
	echo "----- Adding sshd to /etc/inetd.conf"
	grep -v ^ssh /etc/inetd.conf 1>/tmp/inetd.conf 2>/dev/null
	echo "#sshd support added below">>/tmp/inetd.conf
		case "$1" in
			"m")
				echo "ssh	stream	tcp	nowait	root	/usr/sbin/sshd sshd -i -q -f /etc/ssh/sshd_config">>/tmp/inetd.conf;;
			"c")
				echo "ssh	stream	tcp	nowait	root	/mnt/cf/sbin/sshd sshd -i -q -f /etc/ssh/sshd_config">>/tmp/inetd.conf;;
			"s")
				echo "ssh	stream	tcp	nowait	root	/mnt/card/sbin/sshd sshd -i -q -f /etc/ssh/sshd_config">>/tmp/inetd.conf;;
		esac
	cp /tmp/inetd.conf /etc/inetd.conf 1>/dev/null 2>&1
	rm -f /tmp/inetd.conf 1>/dev/null 2>&1
fi

echo ""
echo "----- Copying configuration files"

if [ $SERVER = 1 ]; then
	if [ -f /etc/sshd_config ]; then
		echo "Warning: openssh 3.1 has moved the /etc/sshd_config file into /etc/ssh/sshd_config."
		echo "         Please manually edit the /etc/ssh/sshd_config file to reflect old settings."
		echo "         The old sshd_config can be found at /etc/ssh/sshd_config.old"
		mv -f /etc/sshd_config /etc/ssh/sshd_config.old 1>/dev/null 2>&1
	fi
fi

if [ $SERVER = 1 ]; then
	if [ -f /etc/ssh_config ]; then
		echo "Warning: openssh 3.1 has moved the /etc/ssh_config file into /etc/ssh/ssh_config."
		echo "         Please manually edit the /etc/ssh/ssh_config file to reflect old settings."
		echo "         The old ssh_config can be found at /etc/ssh/ssh_config.old"
		mv -f /etc/ssh_config /etc/ssh/ssh_config.old 1>/dev/null 2>&1
	fi
fi

if [ $SERVER = 1 ]; then
	if ! [ -f /etc/ssh/sshd_config ]; then
		cp -f sshd_config /etc/ssh 1>/dev/null 2>&1
	else
		echo "       /etc/ssh/sshd_config exists"
	fi
fi

if [ $SERVER = 1 ]; then
	if ! [ -f /etc/ssh/ssh_config ]; then
		cp -f ssh_config /etc/ssh 1>/dev/null 2>&1
	else
		echo "       /etc/ssh/ssh_config exists"
	fi
fi

if [ $CLIENT = 1 ]; then
	if ! [ -f /home/root/.ssh/config ]; then
		cp -f ssh_config /home/root/.ssh/config 1>/dev/null 2>&1
	else
		echo "       /home/root/.ssh/config exists"
	fi
fi

#Copy the banner file, but move an existing one if it exists.
if [ $SERVER = 1 ]; then
	if [ -f /etc/ssh_banner ]; then
		mv /etc/ssh_banner /etc/ssh 1>/dev/null 2>&1
	else
		cp -f ssh_banner /etc/ssh 1>/dev/null 2>&1
	fi
fi

if [ $SERVER = 1 ]; then
	echo ""
	echo "----- Copying server programs"
	cp -f sshd $PREFIX/sbin 1>/dev/null 2>&1
	cp -f sftp-server $PREFIX/sbin 1>/dev/null 2>&1
fi

if [ $CLIENT = 1 ]; then
	echo ""
	echo "----- Copying client programs"
	cp -f ssh $PREFIX/bin 1>/dev/null 2>&1
	cp -f ssh-keygen $PREFIX/bin 1>/dev/null 2>&1
	cp -f ssh-agent $PREFIX/bin 1>/dev/null 2>&1
	cp -f sftp $PREFIX/bin 1>/dev/null 2>&1
	cp -f scp $PREFIX/bin 1>/dev/null 2>&1
	cp -f ssh-add $PREFIX/bin 1>/dev/null 2>&1
fi

if [ $FULL = 1 ]; then
	cp -f ssh-keyscan $PREFIX/bin 1>/dev/null 2>&1
fi

# Adding shared zlib library
echo ""
echo "----- Copying shared libraries"
mkdir $PREFIX/lib 1>/dev/null 2>&1
cp -f libz.so.1.1.4 $PREFIX/lib 1>/dev/null 2>&1
ln -s $PREFIX/lib/libz.so.1.1.4 /usr/lib/libz.so.1 1>/dev/null 2>&1
ln -s $PREFIX/lib/libz.so.1.1.4 /usr/lib/libz.so.1.1.4 1>/dev/null 2>&1


if [ $SERVER = 1 ]; then
	echo ""
	echo "----- Generating Server Keys"

	# Move existing openssh older than 3.1 keys into the /etc/ssh directory if they exist.
	# This is used when upgrading openssh.
	mv -f ssh_*_key* ./ssh 1>/dev/null 2>&1

# Uncomment these lines if you want an SSH V1 host key.  Since SSH V1 is disabled, we don't need a host key.
#	if ! [ -f "/etc/ssh/ssh_host_key" ]; then
#		echo "------- Generating RSA key for SSH v1"
#		./ssh-keygen -N "" -q -t rsa1 -f /etc/ssh/ssh_host_key  1>/dev/null 2>&1
#	else
#		echo "      /etc/ssh/ssh_host_key exists"
#	fi

	if ! [ -f "/etc/ssh/ssh_host_rsa_key" ]; then
		echo "------- Generating RSA key for SSH v2"
		./ssh-keygen -N "" -q -t rsa -f /etc/ssh/ssh_host_rsa_key  1>/dev/null 2>&1
	else
		echo "      /etc/ssh/ssh_host_rsa_key exists"
	fi

	if ! [ -f "/etc/ssh/ssh_host_dsa_key" ]; then
		echo "------- Generating DSA key for SSH v2"
		./ssh-keygen -N "" -q -t dsa -f /etc/ssh/ssh_host_dsa_key  1>/dev/null 2>&1
	else
		echo "      /etc/ssh/ssh_host_dsa_key exists"
	fi
fi

echo ""
echo "----- Generating User Keys"

if ! [ -f "/home/root/.ssh/identity" ]; then
	echo "------- Generating RSA key for SSH v1"
	./ssh-keygen -t rsa1 -f /home/root/.ssh/identity
else
	echo "      $HOME/.ssh/identity exists"
fi

if ! [ -f "/home/root/.ssh/id_rsa" ]; then
	echo "------- Generating RSA key for SSH v2"
	./ssh-keygen -t rsa -f /home/root/.ssh/id_rsa
else
	echo "      $HOME/id_rsa exists"
fi

if ! [ -f "/home/root/.ssh/id_dsa" ]; then
	echo "------- Generating DSA key for SSH v2"
	./ssh-keygen -t dsa -f /home/root/.ssh/id_dsa
else
	echo "      $HOME/.ssh/id_dsa exists"
fi

if [ $FULL = 1 ]; then
	echo ""
	echo "----- Installing HTML version of man pages"
	mkdir /home/root/Documents/OpenSSH 1>/dev/null 2>&1
	cp -f *.htm /home/root/Documents/OpenSSH  1>/dev/null 2>&1
	cp -f readme /home/root/Documents/OpenSSH 1>/dev/null 2>&1
	cp -f README.Zaurus /home/root/Documents/OpenSSH 1>/dev/null 2>&1
fi

# Update local search path.  This is redundant since we symlink binaries to /usr/bin and /usr/sbin, but it can't hurt
echo ""
echo "----- Updating search path"
case "$1" in
	"c")
		echo "PATH=$PATH:/mnt/cf/bin:/mnt/cf/sbin">>/home/root/.profile;;
	"s")
		echo "PATH=$PATH:/mnt/card/bin:/mnt/card/sbin">>/home/root/.profile;;
esac

echo ""
echo "----- Setting file permissions"
if [ $CLIENT = 1 ]; then
	chmod 755 $PREFIX\bin\ssh 1>/dev/null 2>&1
	chmod 755 $PREFIX\bin\ssh-keygen 1>/dev/null 2>&1
	chmod 755 $PREFIX\bin\ssh-agent 1>/dev/null 2>&1
	chmod 755 $PREFIX\bin\ssh-add 1>/dev/null 2>&1
	chmod 755 $PREFIX\bin\scp 1>/dev/null 2>&1
	chmod 755 $PREFIX\bin\sftp 1>/dev/null 2>&1
	chmod 644 /etc/ssh/ssh_config 1>/dev/null 2>&1
	chmod 600 /home/root/.ssh/config 1>/dev/null 2>&1
	chmod 600 /home/root/.ssh/identity 1>/dev/null 2>&1
	chmod 600 /home/root/.ssh/id_rsa 1>/dev/null 2>&1
	chmod 600 /home/root/.ssh/id_dsa 1>/dev/null 2>&1
	chmod 644 /home/root/.ssh/identity.pub 1>/dev/null 2>&1
	chmod 644 /home/root/.ssh/id_rsa.pub 1>/dev/null 2>&1
	chmod 644 /home/root/.ssh/id_dsa.pub 1>/dev/null 2>&1
fi

if [ $SERVER = 1 ]; then
	chmod 755 $PREFIX\sbin\sshd 1>/dev/null 2>&1
	chmod 755 $PREFIX\sbin\sftp-server 1>/dev/null 2>&1
	chmod 600 /etc/ssh/sshd_config 1>/dev/null 2>&1
	chmod 600 /etc/ssh/ssh_host_key 1>/dev/null 2>&1
	chmod 600 /etc/ssh/ssh_host_rsa_key 1>/dev/null 2>&1
	chmod 600 /etc/ssh/ssh_host_dsa_key 1>/dev/null 2>&1
	chmod 644 /etc/ssh/ssh_host_key.pub 1>/dev/null 2>&1
	chmod 644 /etc/ssh/ssh_host_rsa_key.pub 1>/dev/null 2>&1
	chmod 644 /etc/ssh/ssh_host_dsa_key.pub 1>/dev/null 2>&1
	chmod 644 /etc/ssh/ssh_banner 1>/dev/null 2>&1
fi

if [ $FULL = 1 ]; then
	chmod 755 $PREFIX\bin\ssh-keyscan 1>/dev/null 2>&1
fi

echo ""
echo "----- Creating symbolic links"
if [ $CLIENT = 1 ]; then
	ln -s $PREFIX/bin/ssh /usr/bin/ssh 1>/dev/null 2>&1
	ln -s $PREFIX/bin/ssh-keygen /usr/bin/ssh-keygen 1>/dev/null 2>&1
	ln -s $PREFIX/bin/scp /usr/bin/scp 1>/dev/null 2>&1
	ln -s $PREFIX/bin/sftp /usr/bin/sftp 1>/dev/null 2>&1
	ln -s $PREFIX/bin/ssh-add /usr/bin/ssh-add 1>/dev/null 2>&1
	ln -s $PREFIX/bin/ssh-agent /usr/bin/ssh-agent 1>/dev/null 2>&1
fi

if [ $SERVER = 1 ]; then
	ln -s $PREFIX/sbin/sshd /usr/sbin/sshd 1>/dev/null 2>&1
	ln -s $PREFIX/sbin/sftp-server /usr/sbin/sftp-server 1>/dev/null 2>&1
fi

if [ $FULL = 1 ]; then
	ln -s $PREFIX/bin/ssh-keyscan /usr/bin/ssh-keyscan 1>/dev/null 2>&1
fi

if [ $SERVER = 1 ]; then
	echo ""
	echo "----- Fixing path on root entry in /etc/passwd"
	grep -v ^root /etc/passwd 1>/tmp/passwd  2>/dev/null
	echo "root:x:0:0:root:/home/root:/bin/sh">>/tmp/passwd
	cp -f /tmp/passwd /etc/passwd 1>/dev/null 2>&1
	rm -f /tmp/passwd  1>/dev/null 2>&1
fi

echo ""
echo "----- Please set a root password for the Zaurus"
passwd

if [ $SERVER = 1 ]; then
	echo ""
	echo "----- Restarting inetd"
	# We pull the process id for the restart of inetd from /var/run/inetd.pid
	kill -HUP `cat /var/run/inetd.pid` 1>/dev/null 2>&1
fi

echo ""
echo "----- Installation complete - ssh ready."

if [ $SERVER = 1 ]; then
	# Do a netstat to verify that sshd is listening on 0.0.0.0
	echo ""
	netstat -a | grep :ssh
fi
echo ""
